package com.youkeda.ykd_audio_recorder;


import android.media.MediaRecorder;
import android.os.Build;
import android.os.Environment;
import io.flutter.plugin.common.MethodChannel.Result;

import android.util.Log;
import java.io.File;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.PluginRegistry;


/** YkdAudioRecorderPlugin */
public class YkdAudioRecorderPlugin implements MethodCallHandler {
  //录音API
  private MediaRecorder mMediaRecorder;
  //录音开始时间与结束时间
  public long startTime, endTime;
  //录音所保存的文件
  private File mAudioFile;
  //文件列表数据
  private List<FileBean> dataList;
  //录音文件保存位置
  public String mFilePath = Environment.getExternalStorageDirectory().getAbsolutePath() + "/audio/";


  public String mSavePath;

  Map map;

  /** Plugin registration. */
  public static void registerWith(PluginRegistry.Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "ykd_audio_recorder");
    channel.setMethodCallHandler(new YkdAudioRecorderPlugin());

    Log.d("-----", "=========");


    if(com.youkeda.ykd_audio_recorder.CustomPhoneStateListener.status){
      channel.invokeMethod("interrupt","END");
    }else {
      channel.invokeMethod("interrupt","BEGIN");

    }
  }




  @Override
  public void onMethodCall(MethodCall call, Result result) {

    Map<String, Object> startMap = new HashMap<>();
    Log.d("******", call.method);

    if (call.method.equals("start")) {
      Object arguments = call.arguments;
      map = (HashMap) arguments;

      this.startRecord((String) map.get("path"));
      startMap.put("success",true);
      result.success(startMap);

    } else if (call.method.equals("stop")) {
      this.stopRecord();
      Map<String,Object> stringObjectMap=new HashMap<>();
        stringObjectMap.put("path",this.mFilePath);
      Date date = new Date(this.endTime-this.startTime);
      SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
      String startTime = sdf.format(date);

      stringObjectMap.put("duration",this.endTime-this.startTime);
      System.out.println(date+"22222");
      System.out.println(this.endTime-this.startTime+"111111111");
      result.success(stringObjectMap);

    } else if (call.method.equals("pause")) {

      this.pause();


    } else if (call.method.equals("getVolumn")) {
      double volumn=   this.getVolumn();
      result.success(volumn);
    } else if (call.method.equals("resume")) {

      this.resume();
    }


  }
  /**
   * @description 开始进行录音
   *
   */
  public void startRecord(String savePath) {

    mSavePath = savePath;



    //播放前释放资源
    releaseRecorder();
    //执行录音操作
    recordOperation();
  }



  /**
   * @description 录音操作
   * @author ldm
   *
   */
  private void recordOperation() {
      //创建录音文件,.m4a为MPEG-4音频标准的文件的扩展名
      mAudioFile = new File(mFilePath + System.currentTimeMillis() + ".m4a");


    //创建MediaRecorder对象
    mMediaRecorder = new MediaRecorder();
    //创建父文件夹
    mAudioFile.getParentFile().mkdirs();
    try {
      //创建文件
      mAudioFile.createNewFile();
      //配置mMediaRecorder相应参数
      //从麦克风采集声音数据
      mMediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
      //设置保存文件格式为MP4
      mMediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4);
      //设置采样频率,44100是所有安卓设备都支持的频率,频率越高，音质越好，当然文件越大
      mMediaRecorder.setAudioSamplingRate(44100);
      //设置声音数据编码格式,音频通用格式是AAC
      mMediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AAC);
      //设置编码频率
      mMediaRecorder.setAudioEncodingBitRate(96000);
      //设置录音保存的文件
      mMediaRecorder.setOutputFile(mAudioFile.getAbsolutePath());
      //开始录音
      mMediaRecorder.prepare();
      mMediaRecorder.start();
      //记录开始录音时间
      startTime = System.currentTimeMillis();
    } catch (Exception e) {
      e.printStackTrace();

    }
  }


  /**
   * @description 结束录音操作
   * @author ldm
   *
   */
  public void stopRecord() {

    //停止录音
    mMediaRecorder.stop();
    //记录停止时间
    endTime = System.currentTimeMillis();
    long s= endTime-startTime;
    //录音时间处理，比如只有大于2秒的录音才算成功
    int time = (int) ((endTime - startTime) / 1000);
    if (time >= 3) {
      //录音成功,添加数据
      FileBean bean = new FileBean();
      bean.setFile(mAudioFile);
      bean.setFileLength(time);
      dataList=new ArrayList<>();
      dataList.add(bean);
    } else {
      mAudioFile = null;
    }
    //录音完成释放资源
    releaseRecorder();
  }

  /**
   * @description 翻放录音相关资源
   * @author ldm
   *
   */
  private void releaseRecorder() {
    if (null != mMediaRecorder) {
      mMediaRecorder.release();
      mMediaRecorder = null;
    }
  }

  /**
   * @description 暂停录音
   * @author ldm
   *
   */
  public void pause(){
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
      mMediaRecorder.pause();
    }
  }

  /**
   * @description 恢复录音
   * @author ldm
   *
   */

  public void resume(){
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
      mMediaRecorder.resume();
    }


  }
  private int BASE = 1;

  public double getVolumn() {
    double db = 0;// 分贝
    if (mMediaRecorder != null) {
      double ratio = (double)mMediaRecorder.getMaxAmplitude() /BASE;
      if (ratio > 1)
        db = 20 * Math.log10(ratio);

      Log.d("11111","分贝值："+db);
    }
    DecimalFormat d = new DecimalFormat("0.0");
    String vl = d.format(db/100);
    double volumn = Double.valueOf(vl);


    return volumn;
  }

}
